@@ -7,7 +7,6 @@ import functools
 import logging
 import os
 import shlex
-import subprocess
 import signal
 import string
 import sys
@@ -15,7 +14,6 @@ from ruamel.yaml import YAML
 import string
 import random
 import copy
-from pprint import pprint
 
 from pathlib import Path
 
@@ -66,14 +64,7 @@ def error(message):
 
 
 def _extract_env_and_vars(parsed):
-    # There has *got* to be a way to do this more simply
     end = {}
-    env = os.environ.copy()
-    vars = {}
-    if 'vars' in parsed:
-        o_vars = parsed['vars']
-    vars = json.loads(json.dumps(o_vars))
-
     #Search through entire file and set all $ as ENV VAR
     with open('dosk.yml') as f:
         datafile = f.readlines()
@@ -91,6 +82,12 @@ def _extract_env_and_vars(parsed):
         except ValueError:
             pass
 
+    env = os.environ.copy()
+    vars = {}
+    if 'vars' in parsed:
+        o_vars = parsed['vars']
+    vars = json.loads(json.dumps(o_vars))
+
     return env, vars
 
 async def env_ject(string):
@@ -115,6 +112,25 @@ async def condition_clean(old_string):
     new = ''
     return new
 
+async def env_string(line):
+    end = {}
+    try:
+        dollar = find(line, '$')
+        for i, d in enumerate(dollar):
+            for idx, letter in enumerate(line):
+                if not letter.isalpha() and idx > dollar[i]:
+                    end[i] = idx
+                    break
+                if idx + 1 == len(line):
+                    end[i] = idx + 1        
+            env_var = line[int(d + 1):int(end[i])]
+    except ValueError:
+        pass
+    
+    out = line.replace('$' + env_var, os.getenv(env_var))
+    print(out)
+    return out
+
 async def load_env(rando):
     with open('{}.json'.format(rando), 'r') as file:
         data=file.read()
@@ -130,14 +146,6 @@ async def load_env(rando):
 def random_generator(size=6, chars=string.ascii_uppercase + string.digits):
     return ''.join(random.choice(chars) for x in range(size))
 
-async def load_envars():
-    with aiofiles.open('temp.json', 'r') as file:
-        data = await file.read()
-
-    env_json = await json.loads(data)
-
-    for i, attribute in aiostream.stream.enumerate(env_json):
-        os.environ[attribute] = await env_json[attribute]
 
 async def _inplace_change(filename, old_string, new_string):
     reading_file = open(filename, "r")
@@ -152,11 +160,14 @@ async def _inplace_change(filename, old_string, new_string):
     writing_file.close()
 
 async def _append_line(filename, line):
-    print(filename, line)
     append_file = open(filename, "a")
     append_file.write("\n" + line)
     append_file.close()
 
+async def watch(stream, prefix=''):
+    async for line in stream:
+        log.info(prefix + line.decode().strip())
+
 @asyncio.coroutine
 def inplace_change(filename, old_string, new_string):
     reading_file = open(filename, "r")
@@ -200,6 +211,7 @@ class TaskDefs(dict):
             if 'dir' in obj:
                 post = obj['dir']
                 directory = directory + '/' + post.replace('$', '')
+                self.run_vars['dir'] =  directory  
 
             if 'condition' in obj:
                 self.run_vars['condition'] = obj['condition'].replace('$', '')
@@ -283,16 +295,12 @@ class Task(object):
         os.chdir(self.dir)
 
         self.proc = await asyncio.create_subprocess_shell( cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE)
-        stdout, stderr = await self.proc.communicate()
-        print(f'[{cmd.replace(env_rnd, "")} exited with {self.proc.returncode}]')
-        if stdout:
-            print(f'[stdout]\n{stdout.decode()}')
-        if stderr:
-            print(f'[stderr]\n{stderr.decode()}')
+        await asyncio.gather(watch(self.proc.stdout, '[stdout]'), watch(self.proc.stderr, '[stderr]'))
 
         await self.proc.wait()
         await load_env(rando)
 
+
 class PythonTask(object):
     def __init__(self, program):
         logging.debug('Creating task with command "{}"'.format(program))
@@ -370,12 +378,11 @@ class SequentialTaskList(TaskList):
                 self.condition = True
 
         if 'replace' in self.vars:
+            os.chdir(self.vars['dir'])
             if isinstance(self.vars['replace'], list):
                 for line in self.vars['replace']:
-                    print(line)
                     old = await env_ject(line[1])
                     new = await env_ject(line[2])
-                    print(old, new)
                     await _inplace_change(line[0], old.format(**self.vars), new.format(**self.vars))
 
         if self.condition:
@@ -409,7 +416,7 @@ class SequentialTaskList(TaskList):
         if 'append' in self.vars:
             if isinstance(self.vars['append'], list):
                 for line in self.vars['append']:
-                    clean = await env_ject(line[1])
+                    clean = await env_string(line[1])
                     await _append_line(line[0], clean.format(**self.vars))
 
 
